import {
  select,
  csv,
  scaleTime,
  scaleLinear,
  axisLeft,
  axisBottom,
  line,
  extent,
  max,
  utcParse,
  utcFormat,
} from 'd3';

export const main = async (container) => {
  // Load and parse data
  const data = await csv(
    'https://raw.githubusercontent.com/Anushapodapati99/SCIENTIFIC-DATA-VISUALIZATION-DATASETS/main/crop_production.csv',
  );

  // Assuming the CSV has a 'Crop_Year' column in YYYY format and 'Production'
  const parseYear = utcParse('%Y');
  data.forEach((d) => {
    d.Crop_Year = parseYear(d.Crop_Year.toString()); // Convert year to Date object
    d.Production = +d.Production; // ensure that the production is a number
  });

  // Sort data by Crop_Year after parsing and converting to proper types
  data.sort((a, b) => a.Crop_Year - b.Crop_Year);

  // Define dimensions
  const margin = {
    top: 20,
    right: 30,
    bottom: 50,
    left: 70,
  };
  const width = 960 - margin.left - margin.right;
  const height = 500 - margin.top - margin.bottom;

  // Define scales
  const xScale = scaleTime()
    .domain(extent(data, (d) => d.Crop_Year))
    .range([0, width]);

  const yScale = scaleLinear()
    .domain([0, max(data, (d) => d.Production)])
    .range([height, 0]);

  // Define line generator
  const lineGen = line()
    .x((d) => xScale(d.Crop_Year))
    .y((d) => yScale(d.Production));

  // Append SVG
  const svg = select(container)
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Append axes
  svg
    .append('g')
    .attr('transform', `translate(0,${height})`)
    .call(axisBottom(xScale).tickFormat(utcFormat('%Y')));

  svg
    .append('g')
    .attr('class', 'y-axis')
    .call(axisLeft(yScale));

  // Unique Crop and State names for dropdown
  const crops = Array.from(
    new Set(data.map((d) => d.Crop)),
  );
  const states = Array.from(
    new Set(data.map((d) => d.State_Name)),
  );

  // Create and append select elements
  const cropSelect = select(container).append('select');
  crops.forEach((crop) => {
    cropSelect.append('option').text(crop);
  });

  const stateSelect = select(container).append('select');
  states.forEach((state) => {
    stateSelect.append('option').text(state);
  });

  // Update chart function
  function updateChart(crop, state) {
    // Filter data for the selected crop and state
    let filteredData = data.filter(
      (d) => d.Crop === crop && d.State_Name === state,
    );

    // Sort the filtered data by Crop_Year
    filteredData.sort((a, b) => a.Crop_Year - b.Crop_Year);

    // Aggregate data if there are multiple records for the same year
    const dataByYear = Array.from(
      d3.group(filteredData, (d) => d.Crop_Year),
      ([year, records]) => ({
        Crop_Year: year,
        Production: d3.sum(records, (r) => r.Production),
      }),
    );

    // Update scales
    yScale.domain([
      0,
      max(dataByYear, (d) => d.Production),
    ]);
    xScale.domain(extent(dataByYear, (d) => d.Crop_Year));

    // Update axes
    svg
      .select('.x-axis')
      .call(
        axisBottom(xScale).tickFormat(d3.utcFormat('%Y')),
      );
    svg.select('.y-axis').call(axisLeft(yScale));

    // Bind data
    const lines = svg
      .selectAll('.line')
      .data([dataByYear], (d) => `${crop}_${state}`);

    lines
      .enter()
      .append('path')
      .attr('class', 'line')
      .merge(lines)
      .attr('d', lineGen)
      .attr('fill', 'none')
      .attr('stroke', 'steelblue')
      .attr('stroke-width', 2);

    lines.exit().remove();

    // Update Y axis
    svg
      .select('.y-axis')
      .transition()
      .duration(1000)
      .call(axisLeft(yScale));
  }

  // Initialize chart
  updateChart(crops[0], states[0]);

  // Dropdown change event handlers
  cropSelect.on('change', function () {
    updateChart(this.value, stateSelect.node().value);
  });

  stateSelect.on('change', function () {
    updateChart(cropSelect.node().value, this.value);
  });
};
